import { Injectable, Injector, Optional, Directive, TemplateRef, ViewContainerRef, Input, ɵɵdefineInjectable, ɵɵinject, APP_INITIALIZER, NgModule } from '@angular/core';
import { first, map, tap, distinctUntilChanged, switchMap, catchError, filter, take, finalize } from 'rxjs/operators';
import { __awaiter, __rest } from 'tslib';
import { makeStateKey, TransferState } from '@angular/platform-browser';
import { Api, ApiError } from '@bravobit/bb-foundation/api';
import { Storage } from '@bravobit/bb-foundation/storage';
import { BehaviorSubject, of, throwError } from 'rxjs';
import { Router } from '@angular/router';
import { HttpErrorResponse, HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';

class AuthConfig {
}

class JwtHelper {
    constructor() {
        this.baseDecodeUnicode = (value) => {
            return decodeURIComponent(atob(value).replace(/(.)/g, (_, p) => {
                let code = p.charCodeAt(0).toString(16).toUpperCase();
                if (code.length < 2) {
                    code = '0' + code;
                }
                return '%' + code;
            }));
        };
        this.parse = (data) => {
            return {
                id: data['iss'] || null,
                type: data['typ'] || null,
                audience: data['aud'] || null,
                issuer: data['iss'] || null,
                subject: data['sub'] || null,
                role: data['role'] || null,
                notValidBefore: this.parseDate(data['nbf']),
                expiresAt: this.parseDate(data['exp']),
                issuedAt: this.parseDate(data['iat'])
            };
        };
        this.parseDate = (epochInSeconds) => {
            if (!epochInSeconds || epochInSeconds <= 0) {
                return null;
            }
            return new Date(epochInSeconds * 1000);
        };
    }
    decode(token) {
        try {
            const json = JSON.parse(this.urlDecode(token.split('.')[1]));
            return this.parse(json);
        }
        catch (_a) {
            return null;
        }
    }
    urlDecode(token) {
        const value = token || '';
        let output = value
            .replace(/-/g, '+')
            .replace(/_/g, '/');
        switch (output.length % 4) {
            case 0:
                break;
            case 2:
                output += '==';
                break;
            case 3:
                output += '=';
                break;
            default:
                throw 'Illegal base64url string!';
        }
        try {
            return this.baseDecodeUnicode(output);
        }
        catch (_a) {
            return atob(output);
        }
    }
}

class AuthMapper {
    constructor() {
        // Routes.
        this.me = 'auth/me';
        this.register = 'auth/register';
        this.resendCode = 'auth/resend';
        this.logout = 'auth/logout';
        this.refresh = 'auth/refresh';
        this.requestPassword = 'auth/reset';
        this.resetPassword = 'auth/reset-password';
    }
    role(data) {
        return (data && data.role) || null;
    }
    toRegister(data) {
        // Retrieve the params.
        const { token, refresh_token, user } = data;
        // Map the data to the correct format.
        return { accessToken: token, refreshToken: refresh_token, user: user };
    }
    toRefresh(data) {
        // Retrieve the params.
        const { token, refresh_token } = data;
        // Map the data to the correct format.
        return { accessToken: token, refreshToken: refresh_token };
    }
}
AuthMapper.decorators = [
    { type: Injectable }
];

class AuthVerifyProvider {
    constructor(_code, _verifyToken, _endpoint = 'auth/verify') {
        this._code = _code;
        this._verifyToken = _verifyToken;
        this._endpoint = _endpoint;
    }
    authenticate(api) {
        return __awaiter(this, void 0, void 0, function* () {
            // Execute API call.
            const { token, refresh_token, user } = yield api.post(this._endpoint, {
                token: this._code,
                verify_token: this._verifyToken
            });
            // Map the data to the correct format.
            return { accessToken: token, refreshToken: refresh_token, user: user };
        });
    }
}

class AuthEmailProvider {
    constructor(_email, _password, _endpoint = 'auth/login') {
        this._email = _email;
        this._password = _password;
        this._endpoint = _endpoint;
    }
    authenticate(api) {
        return __awaiter(this, void 0, void 0, function* () {
            // Execute API call.
            const { token, refresh_token, user, provider, verify_token } = yield api.post(this._endpoint, {
                email: this._email,
                password: this._password
            });
            // Map the data to the correct format.
            return {
                accessToken: token,
                refreshToken: refresh_token,
                user: user,
                provider: provider,
                verifyToken: verify_token
            };
        });
    }
}

class Auth {
    constructor(_storage, _mapper, _injector, _api, _config, _state) {
        this._storage = _storage;
        this._mapper = _mapper;
        this._injector = _injector;
        this._api = _api;
        this._config = _config;
        this._state = _state;
        this._authStateKey = makeStateKey(`bbAuthStateKey`);
        // Data.
        this._user$ = new BehaviorSubject(null);
        // Validate the API was injected.
        if (!this._api) {
            throw new Error('The API is required for this module to work.');
        }
        // Set the storage keys based on the application id.
        const applicationId = this.getConfigString('applicationId') || 'angular';
        this._apiUserStorageKey = `${applicationId}_auth-user`;
        this._apiTokenStorageKey = `${applicationId}_auth-token`;
        this._apiRefreshTokenStorageKey = `${applicationId}_auth-refresh-token`;
        // Fetch the user from the storage.
        const user = this._storage.cookie.get(this._apiUserStorageKey);
        // Cache the user in memory if we have tokens.
        if (!!this.accessToken || !!this.refreshToken) {
            this._user$.next(user);
        }
    }
    initialize() {
        return () => __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c, _d;
            const shouldBootstrap = this.getConfigBoolean('bootstrap', true);
            // Check if the app should bootstrap
            // the authentication.
            if (!shouldBootstrap) {
                return;
            }
            // Only retrieve from the server
            // when an access token is saved.
            if (!this.accessToken && !this.refreshToken) {
                return;
            }
            // Get the key from the server state.
            if (this._state && ((_a = this._state) === null || _a === void 0 ? void 0 : _a.hasKey(this._authStateKey))) {
                const user = (_c = (_b = this._state) === null || _b === void 0 ? void 0 : _b.get(this._authStateKey, null)) !== null && _c !== void 0 ? _c : null;
                return this.setUser(user);
            }
            // Try to fetch the user from the server.
            const user = yield this.me().pipe(first()).toPromise();
            // Set the state if exists.
            if (this._state) {
                (_d = this._state) === null || _d === void 0 ? void 0 : _d.set(this._authStateKey, user !== null && user !== void 0 ? user : null);
            }
            // Save the user in the storage.
            return this.setUser(user);
        });
    }
    get isAuthenticated() {
        return !!this.payload;
    }
    get user() {
        return this._user$.asObservable();
    }
    get accessToken() {
        return this._storage.cookie.get(this._apiTokenStorageKey);
    }
    get refreshToken() {
        return this._storage.cookie.get(this._apiRefreshTokenStorageKey);
    }
    get payload() {
        return new JwtHelper().decode(this.accessToken);
    }
    me() {
        return this._api.get(this._mapper.me);
    }
    signIn(provider, as) {
        return __awaiter(this, void 0, void 0, function* () {
            const _a = yield provider.authenticate(this._api), { accessToken, refreshToken, user } = _a, result = __rest(_a, ["accessToken", "refreshToken", "user"]);
            // Check if the role matches.
            if (as && !as.includes(this._mapper.role(user))) {
                throw new Error('Invalid role.');
            }
            // Validate if the provider is one of the available
            // providers then return the user object and the provider.
            const apiProvider = (result && result.provider) || null;
            const apiVerifyToken = (result && result.verifyToken) || null;
            const availableProviders = ['email', 'sms', 'totp'];
            if (availableProviders.includes(apiProvider)) {
                return { user, provider: apiProvider, verifyToken: apiVerifyToken };
            }
            // Set the tokens in storage.
            this.setTokens(accessToken, refreshToken);
            // Set the user in storage.
            this.setUser(user);
            // Return the user.
            return { user };
        });
    }
    signInWithEmail(email, password, as) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.signIn(new AuthEmailProvider(email, password), as);
        });
    }
    signInWithVerifyCode(code, verifyToken) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.signIn(new AuthVerifyProvider(code, verifyToken));
        });
    }
    resendVerifyCode(verifyToken) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._api.post(this._mapper.resendCode, {
                verify_token: verifyToken
            });
        });
    }
    register(data, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // Execute API call.
            const result = yield this._api.post(this._mapper.register, data, options);
            // Map to the correct response.
            const { accessToken, refreshToken, user } = this._mapper.toRegister(result);
            // Set the tokens in storage.
            this.setTokens(accessToken, refreshToken);
            // Set the user in storage.
            this.setUser(user);
            // Return the user.
            return user;
        });
    }
    logout() {
        try {
            // Log the user out.
            this._api.get(this._mapper.logout, { headers: { Authorization: this.refreshToken } })
                .toPromise().then(_ => _);
        }
        catch (_a) {
            // Do nothing because the tokens will be deleted anyways from the storage.
        }
        // Delete the tokens from the session.
        return this.clear();
    }
    refresh() {
        // If the refresh token does
        // not exist just return an observable of null.
        if (!this.refreshToken) {
            return of(null);
        }
        // Perform the refresh call.
        const scheme = this.getConfigString('schema', 'Bearer');
        return this._api.get(this._mapper.refresh, {
            headers: {
                'Authorization': `${scheme} ${this.refreshToken}`,
                'X-No-Authorization': 'yes'
            }
        }).pipe(map(data => this._mapper.toRefresh(data)), tap(({ accessToken, refreshToken }) => this.setTokens(accessToken, refreshToken)), map(({ accessToken }) => accessToken));
    }
    requestPassword(email, extraParams = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._api.post(this._mapper.requestPassword, Object.assign(Object.assign({}, extraParams), { email }));
            }
            catch (error) {
                throw error;
            }
        });
    }
    resetPassword(token, newPassword, extraParams = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this._api.post(this._mapper.resetPassword, Object.assign(Object.assign({}, extraParams), { token, password: newPassword }));
            }
            catch (error) {
                throw error;
            }
        });
    }
    guard(type, redirectUrl) {
        let newUrl = null;
        // Get the correct new url.
        switch (type) {
            case 'authenticated':
                newUrl = this.getConfigString('loggedInUrl', null);
                break;
            case 'unauthenticated':
                newUrl = this.getConfigString('redirectUrl', null);
                break;
        }
        // Append a redirect url if the user is deemed
        // unauthenticated.
        if (type === 'unauthenticated') {
            const setRedirectOnFailedAuth = this.getConfigBoolean('setRedirectOnFailedAuth', true);
            if (setRedirectOnFailedAuth && redirectUrl && newUrl) {
                newUrl += `?redirectUrl=${redirectUrl}`;
            }
        }
        // Parse the url if it exists.
        if (this.router && newUrl) {
            return this.router.parseUrl(newUrl);
        }
        // Return false if the user is not allowed.
        return false;
    }
    setTokens(accessToken, refreshToken) {
        // Set the token if the token exists.
        if (accessToken) {
            const jwtAccessToken = new JwtHelper().decode(accessToken);
            if (jwtAccessToken) {
                this._storage.cookie.set(this._apiTokenStorageKey, accessToken, {
                    expires: jwtAccessToken === null || jwtAccessToken === void 0 ? void 0 : jwtAccessToken.expiresAt
                });
            }
        }
        // Set the refresh token if the refresh token exists.
        if (refreshToken) {
            const jwtRefreshToken = new JwtHelper().decode(refreshToken);
            if (jwtRefreshToken) {
                this._storage.cookie.set(this._apiRefreshTokenStorageKey, refreshToken, {
                    expires: jwtRefreshToken === null || jwtRefreshToken === void 0 ? void 0 : jwtRefreshToken.expiresAt
                });
            }
        }
    }
    setUser(user) {
        // Validate the user exists.
        if (!user) {
            return;
        }
        // Push the next user.
        this._user$.next(user);
        // Make the user data valid for one year.
        const date = new Date();
        date.setFullYear(date.getFullYear() + 1);
        // Save the user in the storage.
        return this._storage.cookie.set(this._apiUserStorageKey, user, {
            expires: new Date(date.getTime())
        });
    }
    clear() {
        // Delete the items from the local storage.
        this._storage.cookie.remove(this._apiUserStorageKey);
        this._storage.cookie.remove(this._apiTokenStorageKey);
        this._storage.cookie.remove(this._apiRefreshTokenStorageKey);
        // Push an empty user.
        this._user$.next(null);
    }
    get router() {
        return this._injector.get(Router);
    }
    getConfigBoolean(name, defaultValue = false) {
        return (this._config && this._config[name]) === undefined
            ? defaultValue
            : this._config[name];
    }
    getConfigString(name, defaultValue = null) {
        return (this._config && this._config[name]) === undefined
            ? defaultValue
            : this._config[name];
    }
}
Auth.decorators = [
    { type: Injectable }
];
Auth.ctorParameters = () => [
    { type: Storage },
    { type: AuthMapper },
    { type: Injector },
    { type: Api, decorators: [{ type: Optional }] },
    { type: AuthConfig, decorators: [{ type: Optional }] },
    { type: TransferState, decorators: [{ type: Optional }] }
];

class BbAuthenticated {
    constructor(_auth, _template, _viewContainerRef) {
        this._auth = _auth;
        this._template = _template;
        this._viewContainerRef = _viewContainerRef;
    }
    ngOnInit() {
        this._subscription = this._auth.user.pipe(map(user => !!user), distinctUntilChanged()).subscribe(isAuthenticated => {
            if (!isAuthenticated) {
                return this._viewContainerRef.clear();
            }
            this._viewContainerRef.createEmbeddedView(this._template);
        });
    }
    ngOnDestroy() {
        var _a;
        (_a = this._subscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    }
}
BbAuthenticated.decorators = [
    { type: Directive, args: [{
                selector: '[bbAuthenticated]'
            },] }
];
BbAuthenticated.ctorParameters = () => [
    { type: Auth },
    { type: TemplateRef },
    { type: ViewContainerRef }
];

class BbRole {
    constructor(_auth, _template, _viewContainerRef) {
        this._auth = _auth;
        this._template = _template;
        this._viewContainerRef = _viewContainerRef;
        this.getAllowedRoles = (value) => {
            return Array.isArray(value) ? value : [value];
        };
    }
    ngOnInit() {
        this._subscription = this._auth.user.pipe(map(user => { var _a; return (_a = user === null || user === void 0 ? void 0 : user.role) !== null && _a !== void 0 ? _a : null; }), distinctUntilChanged()).subscribe(role => {
            const allowedRoles = this.getAllowedRoles(this.bbRole);
            if (!allowedRoles.includes(role)) {
                return this._viewContainerRef.clear();
            }
            this._viewContainerRef.createEmbeddedView(this._template);
        });
    }
    ngOnDestroy() {
        var _a;
        (_a = this._subscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    }
}
BbRole.decorators = [
    { type: Directive, args: [{
                selector: '[bbRole]'
            },] }
];
BbRole.ctorParameters = () => [
    { type: Auth },
    { type: TemplateRef },
    { type: ViewContainerRef }
];
BbRole.propDecorators = {
    bbRole: [{ type: Input }]
};

class Permissions {
    constructor(_auth, _config) {
        this._auth = _auth;
        this._config = _config;
        // Data.
        this._role$ = this._auth.user.pipe(map(user => { var _a; return (_a = user === null || user === void 0 ? void 0 : user.role) !== null && _a !== void 0 ? _a : null; }), distinctUntilChanged());
    }
    has(value) {
        const requiredPermissions = Array.isArray(value)
            ? value
            : [value];
        return this._role$.pipe(map(role => {
            for (const requiredPermission of requiredPermissions) {
                const result = this.validatePermission(requiredPermission, role);
                if (!result) {
                    return false;
                }
            }
            return true;
        }));
    }
    validatePermission(type, role) {
        var _a, _b, _c, _d, _e;
        const permission = (_b = (_a = this.permissions) === null || _a === void 0 ? void 0 : _a[type]) !== null && _b !== void 0 ? _b : null;
        if (!permission) {
            (_c = console === null || console === void 0 ? void 0 : console.warn) === null || _c === void 0 ? void 0 : _c.call(console, `Permissions: Invalid permission requested "${type}".`);
            return false;
        }
        if ((permission === null || permission === void 0 ? void 0 : permission.length) <= 0) {
            (_d = console === null || console === void 0 ? void 0 : console.warn) === null || _d === void 0 ? void 0 : _d.call(console, `Permissions: No roles were set, please add some roles to this permission.`);
            return false;
        }
        // If the permission includes a "*" we allow everything.
        if (permission.includes('*')) {
            return true;
        }
        // Validate the role against the permissions.
        return (_e = permission === null || permission === void 0 ? void 0 : permission.includes(role)) !== null && _e !== void 0 ? _e : false;
    }
    get permissions() {
        var _a, _b;
        return (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.permissions) !== null && _b !== void 0 ? _b : {};
    }
}
Permissions.decorators = [
    { type: Injectable }
];
Permissions.ctorParameters = () => [
    { type: Auth },
    { type: AuthConfig, decorators: [{ type: Optional }] }
];

class BbPermission {
    constructor(_permissions, _templateRef, _viewContainerRef) {
        this._permissions = _permissions;
        this._templateRef = _templateRef;
        this._viewContainerRef = _viewContainerRef;
        // Data.
        this._permission$ = new BehaviorSubject([]);
        this._valid = false;
        this._elseTemplateRef = null;
        this._thenViewRef = null;
        this._elseViewRef = null;
        this.getAllowedPermissions = (value) => {
            return Array.isArray(value) ? value : [value];
        };
        this.assertTemplate = (property, templateRef) => {
            const isTemplateRefOrNull = !!(!templateRef || templateRef.createEmbeddedView);
            if (!isTemplateRefOrNull) {
                throw new Error(`${property} must be a TemplateRef.`);
            }
        };
    }
    set bbPermission(value) {
        const permissions = this.getAllowedPermissions(value);
        this._permission$.next(permissions);
        this.updateView();
    }
    set bbPermissionElse(templateRef) {
        this.assertTemplate('bbPermissionElse', templateRef);
        this._elseTemplateRef = templateRef;
        this.updateView();
    }
    ngOnInit() {
        const check$ = this._permission$.pipe(switchMap(permissions => this._permissions.has(permissions)));
        this._subscription = check$.subscribe(valid => {
            this._valid = valid;
            this.updateView();
        });
    }
    ngOnDestroy() {
        var _a;
        (_a = this._subscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    }
    updateView() {
        if (this._valid) {
            if (!this._thenViewRef) {
                this._viewContainerRef.clear();
                this._elseViewRef = null;
                if (this._templateRef) {
                    this._thenViewRef = this._viewContainerRef.createEmbeddedView(this._templateRef);
                }
            }
        }
        else {
            if (!this._elseViewRef) {
                this._viewContainerRef.clear();
                this._thenViewRef = null;
                if (this._elseTemplateRef) {
                    this._elseViewRef = this._viewContainerRef.createEmbeddedView(this._elseTemplateRef);
                }
            }
        }
    }
}
BbPermission.decorators = [
    { type: Directive, args: [{
                selector: '[bbPermission]'
            },] }
];
BbPermission.ctorParameters = () => [
    { type: Permissions },
    { type: TemplateRef },
    { type: ViewContainerRef }
];
BbPermission.propDecorators = {
    bbPermission: [{ type: Input }],
    bbPermissionElse: [{ type: Input }]
};

class BbAnonymousGuard {
    constructor(_auth) {
        this._auth = _auth;
    }
    canActivate(_, state) {
        return __awaiter(this, void 0, void 0, function* () {
            // Check if the user is authenticated.
            const isAuthenticated = yield this._auth.user.pipe(map(user => !!user), first()).toPromise();
            // If the user is not authenticated it can
            // access the anonymous page.
            if (!isAuthenticated) {
                return true;
            }
            // Return the user back to the authenticated page.
            return this._auth.guard('authenticated', state.url);
        });
    }
    canActivateChild(childRoute, state) {
        return this.canActivate(childRoute, state);
    }
}
BbAnonymousGuard.ɵprov = ɵɵdefineInjectable({ factory: function BbAnonymousGuard_Factory() { return new BbAnonymousGuard(ɵɵinject(Auth)); }, token: BbAnonymousGuard, providedIn: "root" });
BbAnonymousGuard.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
BbAnonymousGuard.ctorParameters = () => [
    { type: Auth }
];

class BbAuthenticatedGuard {
    constructor(_auth) {
        this._auth = _auth;
    }
    canActivate(_, state) {
        return __awaiter(this, void 0, void 0, function* () {
            // Check if the user is authenticated.
            const isAuthenticated = yield this._auth.user.pipe(map(user => !!user), first()).toPromise();
            // If the user is authenticated it can
            // access the authenticated page.
            if (isAuthenticated) {
                return true;
            }
            // The user is not authorized to see that
            // page so un-authorize him.
            return this._auth.guard('unauthenticated', state.url);
        });
    }
    canActivateChild(childRoute, state) {
        return this.canActivate(childRoute, state);
    }
}
BbAuthenticatedGuard.ɵprov = ɵɵdefineInjectable({ factory: function BbAuthenticatedGuard_Factory() { return new BbAuthenticatedGuard(ɵɵinject(Auth)); }, token: BbAuthenticatedGuard, providedIn: "root" });
BbAuthenticatedGuard.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
BbAuthenticatedGuard.ctorParameters = () => [
    { type: Auth }
];

class AuthInterceptor {
    constructor(_auth, _router, _config) {
        this._auth = _auth;
        this._router = _router;
        this._config = _config;
        // Data.
        this.isRefreshing = false;
        this.token$ = new BehaviorSubject(null);
        this.getToken = (request) => {
            // Get the token based on header.
            if (request.headers.has('Authorization')) {
                return request.headers.get('Authorization');
            }
            // Return the default access token.
            return this._auth.accessToken;
        };
        this.composeRequest = (request, token = null) => {
            // If the token is not set, delete the authorization header.
            if (!token) {
                return request.clone({ headers: request.headers.delete('Authorization') });
            }
            // Get the auth scheme from the config,
            // if the scheme was not set use bearer.
            const authScheme = (this._config && this._config.scheme) || 'Bearer';
            // Clone and add the authorization header.
            return request.clone({
                setHeaders: { Authorization: `${authScheme} ${token}` }
            });
        };
    }
    intercept(request, next) {
        // Just return when the user is performing an refresh request.
        if (request.headers.has('X-No-Authorization')) {
            const newRequest = request.clone({ headers: request.headers.delete('X-No-Authorization') });
            return next.handle(newRequest);
        }
        // Compose the new request.
        const newRequest = this.composeRequest(request, this.getToken(request));
        // Handle all errors.
        return next.handle(newRequest).pipe(catchError(error => {
            // Create a custom error.
            const apiError = new ApiError(error.status, error.error);
            // Check if it is an error response.
            if (!(error instanceof HttpErrorResponse)) {
                return throwError(apiError);
            }
            // Handle a 401 error.
            if (error.status === 401) {
                return this.handle401Error(request, next);
            }
            // Just re-throw the error.
            return throwError(apiError);
        }));
    }
    handle401Error(request, next) {
        // If already refreshing wait for the token to complete.
        if (this.isRefreshing) {
            return this.token$.pipe(filter(token => token !== null), take(1), switchMap(token => next.handle(this.composeRequest(request, token))));
        }
        // Set the refreshing to true.
        this.isRefreshing = true;
        this.token$.next(null);
        return this._auth.refresh().pipe(switchMap(newToken => {
            if (!newToken) {
                return throwError('No refresh token was available.');
            }
            this.token$.next(newToken);
            return next.handle(this.composeRequest(request, newToken));
        }), catchError(error => {
            return this.logoutUser(error);
        }), finalize(() => {
            this.isRefreshing = false;
        }));
    }
    logoutUser(error) {
        // Delete the tokens from the session.
        this._auth.clear();
        // Compose the route url.
        const routeUrl = this._config && this._config.redirectUrl;
        // Route back if the user provided a redirect url.
        if (this._router && routeUrl) {
            this._router.navigate([routeUrl]).then(_ => _);
        }
        // Throw the error.
        return throwError(error || 'Invalid refresh token; user was logged out.');
    }
}
AuthInterceptor.decorators = [
    { type: Injectable }
];
AuthInterceptor.ctorParameters = () => [
    { type: Auth },
    { type: Router, decorators: [{ type: Optional }] },
    { type: AuthConfig, decorators: [{ type: Optional }] }
];

const DECLARATIONS_EXPORTS = [
    BbAuthenticated,
    BbRole,
    BbPermission
];
class AuthModule {
    static forRoot(config) {
        return {
            ngModule: AuthModule,
            providers: [
                Auth,
                Permissions,
                { provide: AuthConfig, useValue: config },
                { provide: HTTP_INTERCEPTORS, useClass: AuthInterceptor, multi: true },
                { useFactory: initializeAuth, provide: APP_INITIALIZER, deps: [Auth], multi: true }
            ]
        };
    }
}
AuthModule.decorators = [
    { type: NgModule, args: [{
                imports: [HttpClientModule],
                declarations: [...DECLARATIONS_EXPORTS],
                exports: [...DECLARATIONS_EXPORTS],
                providers: [
                    { provide: AuthMapper, useClass: AuthMapper }
                ]
            },] }
];
function initializeAuth(auth) {
    return auth.initialize();
}

/**
 * Generated bundle index. Do not edit.
 */

export { Auth, AuthConfig, AuthEmailProvider, AuthMapper, AuthModule, AuthVerifyProvider, BbAnonymousGuard, BbAuthenticated, BbAuthenticatedGuard, BbPermission, BbRole, JwtHelper, Permissions, initializeAuth, AuthInterceptor as ɵa };
//# sourceMappingURL=bravobit-bb-foundation-auth.js.map
